#ifdef NIETMEERNODIG
/**************************************************
  Translation to elevation/azimuth:
    Rsat=h+Rearth;
    h1=sin(obs_lat)*sin(sat_lat) +cos(obs_lat)*cos(sat_lat)*cos(sat_lon-obs_lon);
    h=acos(h1);
    gamma=2*asin(sqrt((1-(h1))/2.) );

    dist=sqrt(Rearth*Rearth+Rsat*Rsat-2*Rearth*Rsat*cos(gamma));
    sat_elev=acos((Rearth-Rsat*cos(gamma))/dist)-pi/2

    sat_azim=acos((sin(sat_lat)-cos(h)*sin(obs_lat))/(cos(obs_lat)*sin(h)));

 **************************************************/
#include <math.h>
#include "xtrack.h"
#include "orbit.h"
/*************************************
 * Calculate elevation/azimuth wrt observer position
 *************************************/
void calceleazim(POINT *pos_subsat,ORBIT *orbit,POINT *refpos,DIRECTION *satdir)
{
  float gamma;
  float lon_o,lat_o,lon_s,lat_s;
  float dlon,h,h1;
  float dist,Rsat;
  lon_s=pos_subsat->lon; lat_s=pos_subsat->lat;
  lon_o=refpos->lon;     lat_o=refpos->lat;
  dlon=lon_s-lon_o;
  Rsat=orbit->height+Rearth;
  h1=sin(lat_o)*sin(lat_s) +cos(lat_o)*cos(lat_s)*cos(dlon);
  h=acos(h1);
  gamma=2*asin(sqrt((1-(h1))/2.) );

  dist=sqrt(Rearth*Rearth+Rsat*Rsat-2*Rearth*Rsat*cos(gamma));
  satdir->elev=acos((Rearth-Rsat*cos(gamma))/dist)-PI/2.;

  {
    float tmp=(sin(lat_s)-cos(h)*sin(lat_o))/(cos(lat_o)*sin(h));
    if (tmp>1.) tmp=1.; if (tmp<-1.) tmp=1.;
    satdir->azim=acos(tmp);
  }

  if (satdir->azim<0) satdir->azim+=(PI*2);
  if (dlon<0) satdir->azim=PI*2-satdir->azim;

  satdir->x=atan2(sin(satdir->azim),tan(satdir->elev));
  satdir->y=asin(cos(satdir->azim)*cos(satdir->elev));
}

/*************************************
 * Translate sensor angle to angle position on earth.
 * (angle between line "sat <--> middle point earth" and "pos. earth <--> mp"
 * Needed: sat_height: height satellite
 *         x         : x-pos. (for HRPT: 0 ... 1023)
 *         xh        : half value of x (1023 for HRPT)
 *         alpha_max : max. angle sensor
 * If x=0  ==> alpha=alpha_max ==> beta=beta_max
 * if x=xh ==> alpha=0 ==> beta=0        
 *************************************/
float x2earthangle(float sat_height,            /* height sat */
                   SCANDIR scan_hor,            /* scan direction sensor */
                   int x,int xh,                 
                   float alpha_max)
{
  float alpha,beta,tmp1,tmp2;

/* angle sensor in satellite */
  alpha=(xh-x)*alpha_max/xh;
  if (scan_hor==W_E) alpha*=-1;

/* translate into radius angle */
  tmp1=(sat_height+Rearth)/Rearth;
  tmp2=(tmp1*tmp1*sin(alpha)*sin(alpha));
  beta=acos(tmp1*sin(alpha)*sin(alpha)+
                 cos(alpha)*sqrt(1-tmp2));
  if (alpha<0) beta*=-1;
  return beta;
}
#endif
