#define NW
#define SCR1
#include "xtrack.h"
#include "xtrack_func.h"
#include "defines.h"
extern DBASE *db;
struct tm tm2showtm(struct tm tm,int rev);

static gboolean sched_update(gpointer data)
{
  GtkWidget *widget=(GtkWidget *)data;
  GtkWidget *window=Find_Parent_Window(widget);
  Update_Main_Wait(widget);
  refresh_wnd(window);
  return FALSE;
}


/*************************************
 * Update window (scheduled)
 *************************************/
void Update_Main(GtkWidget *widget)
{
  g_idle_add(sched_update,widget);
}

static RGBI rgb_bck;
void Update_Main_Wait(GtkWidget *widget)
{
  GtkWidget *window=Find_Parent_Window(widget);
  GtkWidget *main_window=Find_Window(window,Lab_Main);
  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  int i;

  db->redraw=TRUE;
  db->redrawing=TRUE;
  if (Get_Enable_Update(drawing_area))
  {
    db->redraw=FALSE;
    Set_Enable_Update(drawing_area,FALSE); /* Prevent re-drawing while drawing */
    draw_back(main_window);
    if (!db->redraw)
    {
      draw_rast(main_window,TRUE);
 //     Set_Adjust(main_window,SEC,"%d",db->glob_tm.tm_sec); /* force drawing and setting values */
    }
    else
    {
      db->redraw=FALSE;
      Update_Main(widget);
    }
    if (rgbi)
    {
      if (rgb_bck.rgbbuf) free(rgb_bck.rgbbuf);
      rgb_bck.bufsize=rgbi->bufsize;
      rgb_bck.rgbbuf=malloc(rgb_bck.bufsize);
      rgb_bck.width =rgbi->width;
      rgb_bck.height=rgbi->height;
      for (i=0; i<rgb_bck.bufsize; i++) *(rgb_bck.rgbbuf+i)=*(rgbi->rgbbuf+i);
    }
    Set_Enable_Update(drawing_area,TRUE);
  }
  db->redrawing=FALSE;
}

gint configure_event_main(GtkWidget         *widget,
                          GdkEventConfigure *event)
{
  GtkWidget *window=Find_Parent_Window(widget);
  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");

  Renew_RGBBuf(drawing_area);
  Update_Main(widget);
  return TRUE;
}

void zoom_inout(float *z,int *offset,float zfact,int w,char io)
{
  if (io=='i')
  {
    (*z)*=zfact;
    if (*z<1.) *z=1.;
    *offset=*offset+(int)(w/(*z)*(zfact-1.)/2.);
  }
  else
  {
    *offset=*offset-(int)(w/(*z)*(zfact-1.)/2.);
    (*z)/=zfact;
  }
}

void key_actions(GtkWidget *widget, GdkEventKey *event)
{
  GtkWidget *drawing_area=Find_Widget(widget,"GTK_DRAWING_AREA");
  GtkWidget *window=Find_Parent_Window(widget);
  int width=drawing_area->allocation.width/db->ppl;
  int height=drawing_area->allocation.height/db->ppl;
  gboolean redraw=FALSE;
#define FIX_ZMFACT 2.

  switch ((guchar)event->keyval)
  {
    case 'f':
      db->ox=0;
      db->oy=0;
      db->zx=db->zy=1.;
      redraw=TRUE;
    break;
    case 'I':
      zoom_inout(&db->zx,&db->ox,FIX_ZMFACT,width,'i');
      zoom_inout(&db->zy,&db->oy,FIX_ZMFACT,height,'i');
      redraw=TRUE;
    break;
    case 'i':
      db->zx*=FIX_ZMFACT;
      db->zy*=FIX_ZMFACT;
      db->ox=R2D(db->refpos.lon)+LONOFFSET - (width-1)/(db->zx*2);
      db->oy=LATOFFSET-R2D(db->refpos.lat) - (height-1)/(db->zy*2);
      redraw=TRUE;
    break;
    case 'O':
      zoom_inout(&db->zx,&db->ox,FIX_ZMFACT,width,'o');
      zoom_inout(&db->zy,&db->oy,FIX_ZMFACT,height,'o');
/* To prevent coredump in clear_wnd. rgb_bck.bufsize lacks rgb->bufsize! */
      rgb_bck.bufsize=0;
      redraw=TRUE;
    break;
    case 'o':
      db->zx/=FIX_ZMFACT;
      db->zy/=FIX_ZMFACT;
      db->ox=R2D(db->refpos.lon)+LONOFFSET - (width-1)/(db->zx*2);
      db->oy=LATOFFSET-R2D(db->refpos.lat) - (height-1)/(db->zy*2);

/* To prevent coredump in clear_wnd. rgb_bck.bufsize lacks rgb->bufsize! */
      rgb_bck.bufsize=0;
      redraw=TRUE;
    break;
    case GDK_Left&0xff:
      db->ox-=(width/(db->zx*4));
      redraw=TRUE;
    break;
    case GDK_Right&0xff:
      db->ox+=(width/(db->zx*4));
      redraw=TRUE;
    break;
    case GDK_Up&0xff:
      db->oy-=(height/(db->zy*4));
      redraw=TRUE;
    break;
    case GDK_Down&0xff:
      db->oy+=(height/(db->zy*4));
      redraw=TRUE;
    break;
  }

  if (redraw)
  {
    db->setscrolflag=TRUE;
    Set_Scroll(window,"hbar",2*180,&db->ox,&db->zx,
                      "vbar",2* 90,&db->oy,&db->zy);
    db->setscrolflag=FALSE;
/*
    Update_Main(window);
    refresh_wnd(window);
*/
  }
}

void mouse_actions(GtkWidget *widget, GdkEventMotion *event)
{
  GdkModifierType state;
  int x,y;
  static gboolean button1_up;
  if (event->is_hint)
  {
    gdk_window_get_pointer(event->window,&x,&y,&state);
  }
  else
  {
    x=event->x;
    y=event->y;
    state=event->state;
  }
  if (state & GDK_BUTTON1_MASK)
  {
    if (!button1_up)
    {
      SAT *sat=nearest(db->sat,x,y);
      if (sat) Set_Entry(widget,LAB_SATN,"%s",sat->satname);
    }
    button1_up=TRUE;
  }
  if (!(state & GDK_BUTTON1_MASK))
  {
    button1_up=FALSE;
  }
}

static guint idle_handler_id=0;

static int repeated_calc(gpointer p)
{
  static int prev_sec;
  struct tm tm;

  GtkWidget *wnd=(GtkWidget *)p;
  db->glob_tm=mom_tm();
  tm=tm2showtm(db->glob_tm,0);
  if (tm.tm_sec!=prev_sec)
    db->glob_ds=0;
  else if (db->glob_ds<9)
    db->glob_ds++;
  prev_sec=tm.tm_sec;

  Set_Adjust(wnd,SEC,"%d",tm.tm_sec);
  Set_Adjust(wnd,MINT,"%d",tm.tm_min);
  Set_Adjust(wnd,UUR,"%d",tm.tm_hour);
  Set_Adjust(wnd,DAG,"%d",tm.tm_mday);
  Set_Adjust(wnd,MND,"%d",tm.tm_mon+1);
  Set_Adjust(wnd,JAAR,"%d",tm.tm_year+1900);
  Set_Entry(wnd,LAB_OU,"%.2f",db->utc_offset);
  do_onepos_all(wnd);
  idle_handler_id=0;
  return 0;
}

static int repeated_calc1(gpointer p)
{
  if (idle_handler_id==0)
  {
    idle_handler_id=g_idle_add(repeated_calc,p);
  }
  return 1;
}


void buttons_callback(GtkWidget *widget, gpointer data)
{
  GtkWidget *wnd=Find_Parent_Window(widget);
  char *name=(char *)data;
  static int to=0;
  if (!strcmp(name,RUN))
  {
    if ((!to) && (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))))
    {
      int msreptime=(int)(db->reptime*1000);
      if (msreptime<=0) msreptime=1;
      if (wnd)
        to=gtk_timeout_add(msreptime,repeated_calc1,(gpointer)wnd);
    }
    else
    {
      if ((to) && (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))))
      {
        gtk_timeout_remove(to); to=0;
      }
    }
  }
  if (!strcmp(name,ASEL))
  {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
      db->auto_select=TRUE;
    else
      db->auto_select=FALSE;
  }
  if (!strcmp(name,LAB_EXT))
  {
    Update_Togglelabel(widget);
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
      db->out_on=TRUE;
      if (db->to_serial)
      
        if (!RS232_Open_Close(1,db->rs232.portnr, db->rs232.speed))
        {
          Set_Button(wnd,LAB_EXT,0);
          db->out_on=FALSE;
          Create_Message("ERROR","Can't open RS232 port.");
//          Create_Info(NULL,"POEP",0,0,NULL,NULL);
        }
    }
    else
    {
      db->out_on=FALSE;
      if (db->to_serial)
        RS232_Open_Close(0,db->rs232.portnr, db->rs232.speed);
    }
  }

  if (!strcmp(name,SEL))
  {
    select_sat(wnd);
  }
  if (!strcmp(name,EKP))
  {
    Create_kepleredt_wnd(wnd);
  }
}

#ifdef NIETGEBRUIKT
static void file_ok_sel(GtkWidget *widget, GtkFileSelection *fs)
{
  GtkWidget *file_window=Find_Parent_Window(widget);
  if (file_selection_get_filename())
  {
    strcpy(db->norad_file,(char *)file_selection_get_filename());
    strcpy(db->pref_noradfile,db->norad_file);
  }

  if (db->fp_norad) fclose(db->fp_norad);
  db->fp_norad=fopen(db->norad_file,"r");
  if (db->fp_norad)
  {
    Close_Fileselect(file_window); 
    fclose(db->fp_norad); db->fp_norad=NULL;
  }
  else
  {
    Create_Message("Error","Can't open file\n%s.\n",db->norad_file);
  }
}
#endif

void menu_callback(GtkWidget *widget, gpointer data)
{
  GtkWidget *wnd=Find_Parent_Window(widget);
  GtkWidget *drawing_area=Find_Widget(widget,"GTK_DRAWING_AREA");
  int width=drawing_area->allocation.width/db->ppl;
  int height=drawing_area->allocation.height/db->ppl;

  char *name=(char *)data;
  gboolean redraw=FALSE;
  if (!strcmp(name,MENU_KF))
  {
    Get_Kepler(wnd);
  }
/*
  if (!strcmp(name,MENU_KF))
  {
    Create_Fileselectf(wnd,"Kepler-file",NULL,file_ok_sel,NULL,NULL,NULL,0,NULL);
  }
*/

  if (!strcmp(name,MENU_SF))
  {
    select_sat(wnd);
  }
  if (!strcmp(name,MENU_EK))
  {
    Create_kepleredt_wnd(wnd);
  }
  if (!strcmp(name,MENU_PR))
  {
    Create_preferences_wnd(wnd);
  }
  if (!strcmp(name,MENU_PD))
  {
    Create_predict_wnd(wnd);
  }
  if (!strcmp(name,MENU_ZMFULL))
  {
    db->zx=1.;
    db->zy=1.;
    redraw=TRUE;
  }
  if (!strcmp(name,MENU_ZMIN))
  {
    db->zx*=FIX_ZMFACT;
    db->zy*=FIX_ZMFACT;
    db->ox=R2D(db->refpos.lon)+LONOFFSET - (width-1)/(db->zx*2);
    db->oy=LATOFFSET-R2D(db->refpos.lat) - (height-1)/(db->zy*2);
    redraw=TRUE;
  }
  if (!strcmp(name,MENU_ZMOUT))
  {
    db->zx/=FIX_ZMFACT;
    db->zy/=FIX_ZMFACT;
    db->ox=R2D(db->refpos.lon)+LONOFFSET - (width-1)/(db->zx*2);
    db->oy=LATOFFSET-R2D(db->refpos.lat) - (height-1)/(db->zy*2);
    redraw=TRUE;
  }
  if (!strcmp(name,MENU_ZMIN2))
  {
    zoom_inout(&db->zx,&db->ox,FIX_ZMFACT,width,'i');
    zoom_inout(&db->zy,&db->oy,FIX_ZMFACT,height,'i');
    redraw=TRUE;
  }
  if (!strcmp(name,MENU_ZMOUT2))
  {
    zoom_inout(&db->zx,&db->ox,FIX_ZMFACT,width,'o');
    zoom_inout(&db->zy,&db->oy,FIX_ZMFACT,height,'o');
    redraw=TRUE;
  }
  if (redraw)
  {
    Set_Scroll(wnd,"hbar",2*180 ,&db->ox,&db->zx,"vbar",2*90,&db->oy,&db->zy);
/*
    Update_Main(wnd);
*/
  }
}

void time_callback(GtkWidget *widget, gpointer data)
{
  char *name=(char *)data;
  struct tm tm=tm2showtm(db->glob_tm,0);
  GtkWidget *wnd=Find_Parent_Window(widget);
  static gboolean dis=FALSE;
  gboolean manual=FALSE;
  if (dis) return;
  dis=TRUE;
  if (!strcmp(name,SEC))
  {
    tm.tm_sec=GTK_ADJUSTMENT(widget)->value;
    manual=TRUE;
  }

  if (!strcmp(name,MINT))
  {
    tm.tm_min=GTK_ADJUSTMENT(widget)->value;
    manual=TRUE;
  }

  if (!strcmp(name,UUR))
  {
    tm.tm_hour=GTK_ADJUSTMENT(widget)->value;
    manual=TRUE;
  }
  if (!strcmp(name,DAG))
  {
    tm.tm_mday=GTK_ADJUSTMENT(widget)->value;
    manual=TRUE;
  }
  if (!strcmp(name,MND))
  {
    tm.tm_mon=GTK_ADJUSTMENT(widget)->value-1;
    manual=TRUE;
  }
  if (!strcmp(name,JAAR))
  {
    tm.tm_year=GTK_ADJUSTMENT(widget)->value-1900;
    manual=TRUE;
  }
  if (manual)
  {
    mktime_ntz(&tm);
  
    db->glob_tm=tm2showtm(tm,1);

    Set_Adjust(wnd,SEC,"%d",tm.tm_sec);
    Set_Adjust(wnd,MINT,"%d",tm.tm_min);
    Set_Adjust(wnd,UUR,"%d",tm.tm_hour);
    Set_Adjust(wnd,DAG,"%d",tm.tm_mday);
    Set_Adjust(wnd,MND,"%d",tm.tm_mon+1);
    Set_Adjust(wnd,JAAR,"%d",tm.tm_year+1900);

  }
  {
    GtkWidget *w;
    if (db->auto_select)
    {
      db->next_sat=get_next_sat(db->sat,&db->glob_tm);
    }
    else
    {
      db->next_sat=NULL;
    }

    if (!db->sat_sel) { dis=FALSE; return; }

    w=Find_Widget(wnd,db->sat_sel->satname);
    if (w) Set_Local_Button(w,SEL_LAB,TRUE);

    do_onepos_all(wnd);
  }
  dis=FALSE;

  return;
}
#include <math.h>

float calcele1(EPOINT *pos_subsat,ORBIT *orbit,EPOINT *refpos)
{
  float gamma;
  float lon_o,lat_o,lon_s,lat_s;
  float dlon,h1;
  float Rsat;
  float elev;
  lon_s=pos_subsat->lon; lat_s=pos_subsat->lat;
  lon_o=refpos->lon;     lat_o=refpos->lat;
  dlon=lon_s-lon_o;
  Rsat=orbit->height+Rearth;
  h1=sin(lat_o)*sin(lat_s) + cos(lat_o)*cos(lat_s)*cos(dlon);
  gamma=2*asin(sqrt((1-(h1))/2.) );
  elev=acos((Rearth-Rsat*cos(gamma))/Rsat)-PI/2.;
  return elev;
}

extern EPOINT sunpos;
void clear_wnd(GtkWidget *wnd)
{
  GtkWidget *drawing_area=Find_Widget(wnd,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  int i;
  float ele=1.;
  EPOINT refpos;
  float klon,klat;
  ORBIT orbit;

  int x,y;
  for (y=0; y<rgbi->height; y++)   // y:== -90...+90
  {
    for (x=0; x<rgbi->width; x++)  // x:== -180...+180
    {
      if ((db->sat) && (db->sat->visible))   // zon 'aan'
      {
        if (!(x%4))
        {
          orbit.height=150000000.*1000.;
          klon=(x*360/rgbi->width)-180;
          klat=-1*(y*180/rgbi->height)+90;
          refpos.lon=D2R(klon);
          refpos.lat=D2R(klat); 
          ele=calcele1(&sunpos,&orbit,&refpos);
        }
      }

      i=(x+y*rgbi->width)*3;
      if (ele<0)
      {
        *(rgbi->rgbbuf+i)=(*(rgb_bck.rgbbuf+i))/1.1;
        *(rgbi->rgbbuf+i+1)=(*(rgb_bck.rgbbuf+i+1))/1.1;
        *(rgbi->rgbbuf+i+2)=(*(rgb_bck.rgbbuf+i+2))/1.1;
      }
      else
      {
        *(rgbi->rgbbuf+i)=*(rgb_bck.rgbbuf+i);
        *(rgbi->rgbbuf+i+1)=*(rgb_bck.rgbbuf+i+1);
        *(rgbi->rgbbuf+i+2)=*(rgb_bck.rgbbuf+i+2);
      }
    }
  }
}
