#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <math.h>
#include <ctype.h>

#include "xtrack.h"
#include "xtrack_func.h"
#include "defines.h"

extern DBASE *db;
void ld_clr(GdkColor *clr,int i,int r,int g,int b)
{
  clr[i].red=r<<8;
  clr[i].green=g<<8;
  clr[i].blue=b<<8;
}

struct tm mom_tm()
{
  time_t t;
  struct tm tm;
  time(&t);
  tm=*gmtime(&t);
  tm.tm_hour+=(int)(db->time_offset);
  tm.tm_min+=(db->time_offset - (int)(db->time_offset))*60;
  tm.tm_sec+=(int)db->time_offset_sec;
  mktime_ntz(&tm);
  return tm;
}

#ifdef NIETGEBUIKT
struct tm local2gmt(struct tm *ltm)
{
  time_t t;
  struct tm tm1,tm2;
  struct tm gmtm;
  time(&t);
  tm1=*gmtime(&t);
  tm2=*localtime(&t);
  gmtm=*ltm;
  gmtm.tm_hour+=(tm2.tm_hour-tm1.tm_hour);
  gmtm.tm_min+=(tm2.tm_min-tm1.tm_min);
  gmtm.tm_hour-=(int)(db->time_offset);
  gmtm.tm_min-=(db->time_offset - (int)(db->time_offset))*60;
  mktime_ntz(&gmtm);
  return gmtm;
}
#endif
void init_db(DBASE *db,char loc)
{
  switch(loc)
  {
    case 'E':
      db->refpos.lon=D2R(-2.);
      db->refpos.lat=D2R(52.);
    break;
    default:
      db->refpos.lon=D2R(5.);
      db->refpos.lat=D2R(52.);
    break;
  }
  strcpy(db->pref_mapfile,DEF_MAPFILE);

  db->elev_hor=0;
  db->ppl=FACT;
  db->zx=db->zy=1.;
  db->satsel_bottom=FALSE;
  db->glob_tm=mom_tm();
  db->genrange_tm.tm_mon=-1;
  db->genrange_tm.tm_mday=1;
  db->reptime=1.;
  db->is_gmtime=TRUE;
  db->rs232.portnr=0;
  db->rs232.speed=19200;
  strcpy(db->rs232.command,"%d,%d,%d");
}



char *timestr(struct tm *tm)
{
  static char s[50],*p;
  strcpy(s,asctime(tm));
  if ((p=strchr(s,'\n'))) *p=0;
  return s+4;
}

char *tmstrsh(struct tm *tm)
{
  static char s[51];
  strftime(s,50,"%d-%m    %H:%M:%S",tm);
  return s;
}

char *tmstrshx(struct tm *tm,char *what)
{
  static char s[51];
  char frmt[20];
  strcpy(frmt,"");
  if (strchr(what,'m')) strcat(frmt,"%d-%m ");
  if (strchr(what,'H')) strcat(frmt,"%H");
  if (strchr(what,'M')) strcat(frmt,":%M");
  if (strchr(what,'S')) strcat(frmt,":%S");
  strftime(s,50,frmt,tm);
  return s;
}

void alloc_err(char *s)
{
  fprintf(stderr,"Mem allocate error!\n");
  if (s) fprintf(stderr,"%s\n",s);
  exit(1);
}


char *dbl_prct(char *p)
{
  static char tmp[1000];
  char *q=tmp;
  *tmp=0;
  for (; *p; p++)
  {
    if (*p=='%') { *q='%'; q++; }
    *q=*p; q++; *q=0;
  }
  return tmp;
}

/* Detect if file exist */
int exist_file(char *fn)
{
  FILE *fp;
  if (!(*fn)) return 0;
  if (!(fp=fopen(fn,"r"))) return 0;
  fclose(fp);
  return 1;
}

/* Allocate and copy. 2 extra bytes allocated: for closing 0 and maybe 1 extra char to add. */
int strcpyd(char **sp,char *s)
{
  if (!(*sp=(char *)malloc(strlen(s)+2))) return 1;
  strcpy(*sp,s);
  return 0;
}

/* Allocate and concat. 2 extra bytes allocated: for closing 0 and maybe 1 extra char to add. */
int strcatd(char **sp,char *s)
{
  if (!*sp)
  {
    strcpyd(sp,s);
    return 0;
  }
  
  if (!(*sp=(char *)realloc(*sp,strlen(*sp)+strlen(s)+2))) return 1;
  strcat(*sp,s);
  return 0;
}
void strcatc(char *s,char c)
{
  *(s+strlen(s)+1)=0;
  *(s+strlen(s))=c;
}
#define RCHAR(l) l[strlen(l)-1]
void finish_path(char *path)
{
  if (!path) return;
  if (RCHAR(path) != DIR_SEPARATOR)
    strcatc(path,DIR_SEPARATOR);
}

int exist_file_in_dir(char *dir,char *fn)
{
  char *tmp=NULL;
  int res;
  strcpyd(&tmp,dir);
  finish_path(tmp);
  strcatd(&tmp,fn);
  res=exist_file(tmp);
  free(tmp);
  return res;
}
