#define USE_GTK
#include <string.h>
#include "xtrack.h"
#include "sattrack.h"
#include "xtrack_func.h"
#include "defines.h"
extern DBASE *db;

TRACK *Create_Track(TRACK *track)
{
  TRACK *track_nw;

  track_nw=calloc(1,sizeof(TRACK));
  if (!track_nw) exit(1); // alloc_err("Create_Track");

  if (track)
  {
    while (track->next) track=track->next;

    track->next=track_nw;
    track_nw->prev=track;
  }
  return track_nw;
}

void remove_tracks(TRACK *track)
{
  while (track)
  {
    if (track->next)
    {
      track=track->next;
      free(track->prev);
    }
    else
    {
      free(track);
      break;
    }
  }
}
typedef enum {mode_unk=0,search_up,search_down,search_maxelev} SEARCH_MODE;


/* Search for satellite passing horizon */
static gboolean search_horizon(SAT *sat,int sec_incr,
                        SEARCH_MODE search_mode,
                        struct tm *start_tm,  /* start time -> calc. time */
                        EPOINT *pos_subsat,float *mom_elev,EPOINT *refpos,float elev_hor)
{
  DIRECTION satdir;
  float prev_elev=-10;
  int i;
  long stopsec=24*60*60; // sec_incr;
  for (i=0; i<stopsec; i+=sec_incr)
  {
    mktime_ntz(start_tm);
/* calc. mom. position */
    calc_satpos(*start_tm,pos_subsat,&satdir,refpos,sat,db->x_at_disc);
    if (mom_elev) *mom_elev=satdir.elev;

    switch(search_mode)
    {
      case search_up:
        if (satdir.elev>=elev_hor) return TRUE;
      break;

      case search_maxelev:
        if (mom_elev)
        {
          if (*mom_elev < prev_elev) return TRUE;
          prev_elev=*mom_elev;
        }
      break;
      case search_down: /* Also case search_maxelev! */
        if (satdir.elev<elev_hor) return TRUE;
      break;
      default: break;
    }
    start_tm->tm_sec+=sec_incr;
  }
  return FALSE;
}

/****************************************************************
  Predict sat. overkomst
  sat         predict for this sat.
  start_tm    start searching from this time; returns up-time
  stop_tm     if !=NULL this will return down-time 
  tr          gets info about sat. track while above horizon
 ****************************************************************/
gboolean predict(SAT *sat,struct tm *start_tm,struct tm *stop_tm,TRACK *tr,EPOINT *refpos,float elev_hor,float elev_det)
{
float elex;
sat->max_elev=0;
/* Search 'down'; find last sat. pass */
aap:;
  if (!search_horizon(sat,30,search_down,start_tm,&tr->up_pos,&elex,refpos,elev_hor))
    return FALSE;

/* ========== Search satellite Up ========== */
/* Seach 'Up' fast */
  if (!search_horizon(sat,30,search_up,start_tm,&tr->up_pos,&elex,refpos,elev_hor))
    return FALSE;

/* Go back 1 min and search 'Up' precise */
  start_tm->tm_min--;
  start_tm->tm_sec++;
  if (!search_horizon(sat,1,search_up,start_tm,&tr->up_pos,&elex,refpos,elev_hor))
    return FALSE;
  tr->up_time=*start_tm;

/* Stop if no 'stop_time' requested */
  if (!stop_tm) return TRUE;

/* ========== Search satellite max. elevation ========== */
/* Asume 'Up', search 'Max. elev' and determine max. elevation */
  *stop_tm=*start_tm;
  if (!search_horizon(sat,30,search_maxelev,stop_tm,&tr->maxelev_pos,&tr->max_elev,refpos,0.))
    return FALSE;

/* Go back 1 min and search 'Max. elev' precise */
  stop_tm->tm_min--;
  stop_tm->tm_sec++;
  if (!search_horizon(sat,1,search_maxelev,stop_tm,&tr->maxelev_pos,&tr->max_elev,refpos,0.))
    return FALSE;

  tr->maxelev_time=*stop_tm;
  tr->pass_e1_w0=(tr->maxelev_pos.lon > refpos->lon? 1 : 0);
  sat->pass_e1_w0=tr->pass_e1_w0;

/* ========== Search satellite Down ========== */
/* Asume 'Up', search 'Down' fast */
  if (!search_horizon(sat,30,search_down,stop_tm,&tr->down_pos,NULL,refpos,elev_hor))
    return FALSE;

/* Go back 1 min and search 'Down' precise */
  stop_tm->tm_min--;
  stop_tm->tm_sec++;
  if (!search_horizon(sat,1,search_down,stop_tm,&tr->down_pos,NULL,refpos,elev_hor))
    return FALSE;

  tr->down_time=*stop_tm;
  if (sat->max_elev<=0) sat->max_elev=tr->max_elev;
  if ((tr->max_elev>-10) && (tr->max_elev<elev_det))
  {
    *start_tm=*stop_tm;
    start_tm->tm_min++;
//printf("OVERSLAAN: %f / %f;   %s",tr->max_elev,elev_det,asctime(start_tm));
    goto aap;
  }

/* Ready; both start and stop time determined. */
//printf("XX: maxelev=%f\n",R2D(tr->max_elev));
  return TRUE;
}

