#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "xtrack.h"
#include "xtrack_func.h"

#define PREF_FILE ".xtrackrc"
#define PREF_FILE2 "xtrack.ini"

#define SEPCHAR " \n	"
#if __GTK_WIN32__ == 1
  #define PORTMIN 1
  #define PORTMAX 16
#else
  #define PORTMIN 0
  #define PORTMAX 29
#endif

char *get_fullarg(char *w)
{
  char *p;
  if (*w=='"')
  {
    w++;
    if ((p=strchr(w,'"')))
    {
      *p=0;
    }
    else
    {
      p=strtok(NULL,"\"");
      if (p) sprintf(w,"%s %s",w,p);
    }
  }
  return w;
}

void get_dirs(char *progname,char **cur_dir,char **prog_dir,char **home_dir)
{
  if (*cur_dir)  free(*cur_dir);  *cur_dir=NULL;
  if (*prog_dir) free(*prog_dir); *prog_dir=NULL;
  if (*home_dir) free(*home_dir); *home_dir=NULL;

  *cur_dir=g_get_current_dir();
  *prog_dir=get_path(progname);
  *home_dir=(char *)g_get_home_dir();
  #if __GTK_WIN32__ == 1
  {
    if (!(*home_dir))
    {
      if (!strncmp(*cur_dir+1,":\\",2))
      {
        *home_dir=malloc(10);
        strncpy(*home_dir,*cur_dir,2);
        *((*home_dir)+2)=0;
      }
    }
  }
  #endif
}

int search_file(char *rootfile,char *file,
                 char *cur_dir,char *home_dir,char *prog_dir)
{
/* Search pref-file */
  if (cur_dir)
    sprintf(file,"%s%c%s",cur_dir,DIR_SEPARATOR,rootfile);

  if (!(exist_file(file)))
  {
    *file=0;
    if (home_dir)
      sprintf(file,"%s%c%s",home_dir,DIR_SEPARATOR,rootfile);

    if (!(exist_file(file)))
    {
      *file=0;
      if (prog_dir)
        sprintf(file,"%s%c%s",prog_dir,DIR_SEPARATOR,rootfile);

      if (!(exist_file(file)))
      {
        *file=0;
      }
    }
  }
  if (!*file) return 0;
  return 1;
}

void Read_Prefs(char *progname,DBASE *db)
{
  FILE *fp;
  SAT *sat=NULL;
  char l[MAX_FILENAME_LENGTH],*w1,*w2;
  char satname[100];

/* Load directory names: current, program location etc. */
  get_dirs(progname,&db->cur_dir,&db->prog_dir,&db->home_dir);
  if (!(search_file(PREF_FILE2,db->used_preffile,
              db->cur_dir,db->home_dir,db->prog_dir)))
    search_file(PREF_FILE,db->used_preffile,
              db->cur_dir,db->home_dir,db->prog_dir);
  if (!(fp=fopen(db->used_preffile,"r"))) return;

  while (fgets(l,MAX_FILENAME_LENGTH,fp))
  {
    if (((w1=strchr(l,':'))) && (*(w1+1)!='\\')) *w1=' ';
    w1=strtok(l,SEPCHAR);    if (!w1) continue;
    w2=strtok(NULL,SEPCHAR); if (!w2) continue;

    if ((!strcmp(w1,"Norad_file")) && (!*db->norad_file))
    {
      char sep[10];
      sprintf(sep,"%c.",DIR_SEPARATOR);
      strcpy(db->pref_noradfile,w2);

      #if __GTK_WIN32__ == 1
        if (*(db->pref_noradfile+1)==':')
          strcpy(db->norad_file,db->pref_noradfile);
        else
      #endif
        if (strchr(sep,*db->pref_noradfile))
          strcpy(db->norad_file,db->pref_noradfile);
        else
          search_file(db->pref_noradfile,db->norad_file,
                      db->cur_dir,db->home_dir,db->prog_dir);

    }
/*
    if (!strcmp(w1,"Hour_Offset"))  db->hour_offset=atoi(w2);
*/
    if (!strcmp(w1,"Hour_Offset"))  db->time_offset=atof(w2); 
    if (!strcmp(w1,"Sec_Offset"))   db->time_offset_sec=atof(w2); 
    if (!strcmp(w1,"Localtime_UTCOffset"))  db->utc_offset=atof(w2);

    if (!strcmp(w1,"Observer_Lon")) db->refpos.lon=D2R(atof(w2));
    if (!strcmp(w1,"Observer_Lat")) db->refpos.lat=D2R(atof(w2));
    if (!strcmp(w1,"Rotortype"))    db->use_xy=(strstr(w2,"X-Y")? 1 : 0);
    if (!strcmp(w1,"X_at_Disc"))    db->x_at_disc=atoi(w2);
    if (!strcmp(w1,"X_at_Dish"))    db->x_at_disc=atoi(w2);
    if (!strcmp(w1,"Elev_detect"))  db->elev_det=D2R((float)atoi(w2));
    if (!strcmp(w1,"Map_file"))
    {
      char sep[10];
      sprintf(sep,"%c.",DIR_SEPARATOR);
      strcpy(db->pref_mapfile,w2);

      #if __GTK_WIN32__ == 1
        if (*(db->pref_mapfile+1)==':')             // full path
          strcpy(db->map_file,db->pref_mapfile);
        else
      #endif
        if (strchr(sep,*db->pref_mapfile))          // full path
          strcpy(db->map_file,db->pref_mapfile);
        else
          search_file(db->pref_mapfile,db->map_file,
                      db->cur_dir,db->home_dir,db->prog_dir);
puts(db->map_file);
    }
    if (!strcmp(w1,"Rep_time"))     db->reptime=atof(w2);
    if (!strcmp(w1,"Run_at_start"))
      db->start_now=(!strncmp(w2,"y",1)? TRUE : FALSE);
    if (!strcmp(w1,"Ext_progs"))
      db->ext_on=(!strncmp(w2,"y",1)? TRUE : FALSE);
    if (!strcmp(w1,"Output_pos"))
      db->out_on=(!strncmp(w2,"y",1)? TRUE : FALSE);
    if (!strcmp(w1,"Serial"))
      db->to_serial=(!strncmp(w2,"y",1)? TRUE : FALSE);
    if (!strcmp(w1,"Portnr"))
      db->rs232.portnr=MAX(atoi(w2)-PORTMIN,0);
    if (!strcmp(w1,"Baudrate"))
      db->rs232.speed=atoi(w2);
    if (!strcmp(w1,"Poscmd"))
      strncpy(db->rs232.command,w2,90);

    if (!strcmp(w1,"Prog_up"))      strcpy(db->prog_up,get_fullarg(w2));
    if (!strcmp(w1,"Prog_down"))    strcpy(db->prog_down,get_fullarg(w2));
    if (!strcmp(w1,"Prog_track"))   strcpy(db->prog_track,get_fullarg(w2));
    if (!strcmp(w1,"Prog_trackup")) strcpy(db->prog_trackup,get_fullarg(w2));

    if (!strcmp(w1,"Font_size"))    db->fontsize=atoi(w2);

    if (!strcmp(w1,"Buttonsatsel_position"))
      db->satsel_bottom=(!strcmp(w2,"bottom")? TRUE : FALSE);

    if ((!strcmp(w1,"Satellite")) && (!*db->satname))
    {
      SAT *sat1;
      gboolean selected=FALSE;
      gboolean visible=FALSE;
      w2=get_fullarg(w2);
      strcpy(satname,w2);
      if ((w2=strtok(NULL,SEPCHAR))) selected=atoi(w2);
      if ((w2=strtok(NULL,SEPCHAR))) visible=atoi(w2);

      if ((!(db->fp_norad=fopen(db->norad_file,"r"))))
      { 
        Create_Message("Warning2","Kepler-file '%s' not found.",db->norad_file);
        return;
      }

      sat1=read_msat(db->fp_norad,satname,db->sat);
      if (!sat)
      {
        sat=sat1;
        if (sat1)
        {
          sat1->selected=selected;
          sat1->visible=visible;
        }
      }
      else if (sat1)
      {
        SAT *s;
        for (s=sat; (s) && (s->next); s=s->next);
        s->next=sat1; sat1->prev=s;
        sat1->selected=selected;
        sat1->visible=visible;
      }
      if (db->fp_norad) fclose(db->fp_norad); db->fp_norad=NULL;
    }
  }
  if (sat)
  {
    add_to_db(db,sat,0);
    strcpy(db->satname," ");
  }
  fprintf(stderr,"Preference file %s found.\n",db->used_preffile);

  if ((*db->pref_mapfile) && (!*db->map_file))
  {
      char sep[10];
      sprintf(sep,"%c.",DIR_SEPARATOR);
  #if __GTK_WIN32__ == 1
    if (*(db->pref_mapfile+1)==':')
      strcpy(db->map_file,db->pref_mapfile);
    else
  #endif
    if (strchr(sep,*db->pref_mapfile))
      strcpy(db->map_file,db->pref_mapfile);
    else
      search_file(db->pref_mapfile,db->map_file,
                  db->cur_dir,db->home_dir,db->prog_dir);
  }
}

void Save_Prefs(DBASE *db,gboolean save_sat)
{
  SAT *sat;
  FILE *fp=stdout;
  if (!(fp=fopen(PREF_FILE2,"w")))
  {
    Create_Message("Error","Can't open pref-file %s for write.",PREF_FILE2);
    return;
  }

  fprintf(fp,"Norad_file     %s\n",db->pref_noradfile);
  fprintf(fp,"Hour_Offset  : %3.2f\n",db->time_offset);
  fprintf(fp,"Sec_Offset   : %3.1f\n",db->time_offset_sec);
  fprintf(fp,"Localtime_UTCOffset  : %3.2f\n",db->utc_offset);

/*
  fprintf(fp,"Hour_Offset  : %d\n",db->hour_offset);
*/
  fprintf(fp,"Observer_Lon : %f\n",R2D(db->refpos.lon));
  fprintf(fp,"Observer_Lat : %f\n",R2D(db->refpos.lat));

  fprintf(fp,"Rotortype    : %s\n",(db->use_xy? "X-Y": "Elev-Azim"));
  fprintf(fp,"X_at_Disc    : %d\n",db->x_at_disc);
  fprintf(fp,"Elev_detect  : %d\n",(int)R2D(db->elev_det));

  fprintf(fp,"Map_file     : %s\n",db->pref_mapfile);
  fprintf(fp,"Rep_time     : %.1f\n",db->reptime);
  fprintf(fp,"Run_at_start : %s\n",(db->start_now? "y" : "n"));
  fprintf(fp,"Output_pos   : %s\n",(db->out_on? "y" : "n"));
  fprintf(fp,"Ext_progs    : %s\n",(db->ext_on? "y" : "n"));

  fprintf(fp,"Serial       : %s\n",(db->to_serial? "y" : "n"));
  fprintf(fp,"Portnr       : %d\n",db->rs232.portnr+PORTMIN);
  fprintf(fp,"Baudrate     : %d\n",db->rs232.speed);
  fprintf(fp,"Poscmd       : %s\n",db->rs232.command);
  fprintf(fp,"Prog_up      : \"%s\"\n",db->prog_up);
  fprintf(fp,"Prog_down    : \"%s\"\n",db->prog_down);
  fprintf(fp,"Prog_track   : \"%s\"\n",db->prog_track);
  fprintf(fp,"Prog_trackup : \"%s\"\n",db->prog_trackup);

  fprintf(fp,"Font_size    : %d\n",db->fontsize);
  fprintf(fp,"Buttonsatsel_position: %s\n",
              (db->satsel_bottom?"bottom":"right"));

  if (save_sat)
  {
    for (sat=db->sat; sat; sat=sat->next)
    {
      fprintf(fp,"Satellite: \"%s\" %d %d\n",
                           sat->satname,sat->selected,sat->visible);
    }
  }
  fclose(fp);
}

extern DBASE *db;

#define LAB_SAVE "Save preferences"
#define LAB_CLOSE "Close"
#define LAB_LOPREF "Local preference file"
#define LAB_GLPREF "Used preference file"
#define LAB_ISGM     "Show time as GMT "
#define LAB_POSLON   "!Longitude      "
#define LAB_POSLAT   "!Latitude      "
#define LAB_ELEDET   "!detect level"
#define LAB_XATDISC  "!X at Dish     "
#define LAB_RTYPE     "Rotortype:"
#define LAB_RTYPEB    "Elev-Azim/ X-Y "

#define LAB_PROGTRACK   "!Run always          "
#define LAB_PROGUP      "!Run 1x at sat. up   "
#define LAB_PROGDOWN    "!Run 1x at sat. down "
#define LAB_PROGTRACKUP "!Run during sat. up  "

#define LAB_KEPF "!Kepler file"
#define LAB_MAPF "!Map file   "

#define LAB_POSSATSEL   "Button position"
#define LAB_POSSATSEL_R "Right"
#define LAB_POSSATSEL_B "Bottom"

#define LAB_REPTIM      "Interval [sec]"
#define LAB_START       "Run at program start"
#define LAB_OUTON       "Enable at program start"
#define LAB_EXTPROG     "Use external progs"
#define LAB_SERIAL      "Use serial"
#define LAB_FREQ        "L Freq/H Freq"

#define LAB_USE_EXTPROG "Use ext. prog"
#define LAB_USE_RS232   "Use RS232"

#define LAB_FONTS       "Font size"
#define LAB_OFFSETTM    "!Offset UTC time (hours)"       // als PC-tijd afwijkt van UTC
#define LAB_OFFSETMS    "!Offset UTC time (millisecs)"   //
#define LAB_OFFSETUTC   "!Presented time: offset UTC " // invloed getoonde tijd

#if __GTK_WIN32 == 1
  #define LAB_COMPORT  "!RS232 portnr: COM"
#else
  #define LAB_COMPORT  "!RS232 portnr  "
#endif
#define LAB_COMSPEED "!RS232 baudrate"
#define LAB_COMMAND "!RS232 cmd"

#define LAB_EXAMPLE1 "2 integers"
#define LAB_EXAMPLE2 "2 floats"
#define LAB_EXAMPLE3 "3 integers"
#define LAB_EXAMPLE4 "1 integer, 2 floats"

void callback_pref(GtkWidget *widget,gpointer data)
{
  char *name=(char *)data;
  static char *kfn,*mfn;
  if (!strcmp(name,LAB_OFFSETUTC))
  {
    db->utc_offset=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_OFFSETTM))
  {
    db->time_offset=GTK_ADJUSTMENT(widget)->value;
/*
    db->hour_offset=GTK_ADJUSTMENT(widget)->value;
*/
  }
  if (!strcmp(name,LAB_OFFSETMS))
  {
    db->time_offset_sec=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_ISGM))
  {
    db->is_gmtime=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }
  if (!strcmp(name,LAB_POSLON))
  {
    db->refpos.lon=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_POSLAT))
  {
    db->refpos.lat=D2R(GTK_ADJUSTMENT(widget)->value);
  }

  if (!strcmp(name,LAB_ELEDET))
  {
    db->elev_det=D2R(GTK_ADJUSTMENT(widget)->value);
  }

  if (!strcmp(name,LAB_RTYPEB))
  {
    Update_Togglelabel(widget);
    db->use_xy=Get_Button(widget,LAB_RTYPEB);
    Sense_Button(widget,LAB_XATDISC,db->use_xy);
  }

  if (!strcmp(name,LAB_XATDISC))
  {
    db->x_at_disc=Get_Button(widget,LAB_XATDISC);
  }

  if (!strcmp(name,LAB_REPTIM))
  {
    db->reptime=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_PROGUP))
  {
    strcpy(db->prog_up,Get_Entry(gtk_widget_get_toplevel(widget),LAB_PROGUP));
  }
  if (!strcmp(name,LAB_PROGDOWN))
  {
    strcpy(db->prog_down,Get_Entry(gtk_widget_get_toplevel(widget),LAB_PROGDOWN));
  }
  if (!strcmp(name,LAB_PROGTRACK))
  {
    strcpy(db->prog_track,Get_Entry(gtk_widget_get_toplevel(widget),LAB_PROGTRACK));
  }
  if (!strcmp(name,LAB_PROGTRACKUP))
  {
    strcpy(db->prog_trackup,Get_Entry(gtk_widget_get_toplevel(widget),LAB_PROGTRACKUP));
  }

  if (!strcmp(name,LAB_KEPF))
  {
    kfn=Get_Entry(gtk_widget_get_toplevel(widget),LAB_KEPF);
  }
  if (!strcmp(name,LAB_MAPF))
  {
    mfn=Get_Entry(gtk_widget_get_toplevel(widget),LAB_MAPF);
  }

  if (!strcmp(name,LAB_FONTS))
  {
    db->fontsize=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_POSSATSEL_B))
    db->satsel_bottom=TRUE;
  if (!strcmp(name,LAB_POSSATSEL_R))
    db->satsel_bottom=FALSE;
  if (!strcmp(name,LAB_START))
  {
    db->start_now=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }
  if (!strcmp(name,LAB_OUTON))
  {
    db->out_on=Get_Button(widget,LAB_OUTON);
  }
  if (!strcmp(name,LAB_EXTPROG))
  {
    db->ext_on=Get_Button(widget,LAB_EXTPROG);
  }
  if (!strcmp(name,LAB_SERIAL))
  {
    db->to_serial=Get_Button(widget,LAB_SERIAL);
  }
  if (!strcmp(name,LAB_FREQ))
  {
    Update_Togglelabel(widget);
    db->hres=Get_Button(widget,LAB_FREQ);
  }
  if (!strcmp(name,LAB_COMPORT))
  {
    db->rs232.portnr=GTK_ADJUSTMENT(widget)->value-PORTMIN;
  }
  if (!strcmp(name,LAB_COMSPEED))
  {
    db->rs232.speed=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_COMMAND))
  {
    strcpy(db->rs232.command,Get_Entry(gtk_widget_get_toplevel(widget),LAB_COMMAND));
  }

  if (!strcmp(name,LAB_EXAMPLE1))
  {
    Set_Entry(widget,LAB_COMMAND,"%s","%d,%d");
  }
  if (!strcmp(name,LAB_EXAMPLE2))
  {
    Set_Entry(widget,LAB_COMMAND,"%s","%05.1f,%05.1f");
  }
  if (!strcmp(name,LAB_EXAMPLE3))
  {
    Set_Entry(widget,LAB_COMMAND,"%s","%d,%d,%d");
  }
  if (!strcmp(name,LAB_EXAMPLE4))
  {
    Set_Entry(widget,LAB_COMMAND,"%s","%d,%05.1f,%05.1f");
  }

  if (!strcmp(name,LAB_SAVE))
  {
    if (kfn) strcpy(db->pref_noradfile,kfn);
    if (mfn) strcpy(db->pref_mapfile,mfn);
    Save_Prefs(db,TRUE);
  }
  if (!strcmp(name,LAB_CLOSE))
  {
    GtkWidget *wnd=gtk_widget_get_toplevel(widget);
    kfn=NULL;
    mfn=NULL;
    Close_Window(wnd);
  }
}

#ifdef XXX
typedef struct tm FunctionType(struct tm *);

#define LAB_HOUR "^hour"
#define LAB_MIN "^min"
#define LAB_SEC "^sec"
#define LAB_MSEC "^msec"
#define FUNCID "FUNCTIM"
static GtkWidget *ww;
static void Create_TimeArray_func(GtkWidget *widget, gpointer data)
{
  struct tm tm;
  static int msec;
  static int stop;
  char *name=(char *)data;

  FunctionType* func=gtk_object_get_data(GTK_OBJECT(ww),FUNCID);
  tm=func(NULL);

  if (stop) return;

  tm.tm_sec=Get_Adjust(widget,LAB_SEC);
  tm.tm_min=Get_Adjust(widget,LAB_MIN);
  tm.tm_hour=Get_Adjust(widget,LAB_HOUR);
  if (!strcmp(name,LAB_MSEC))
  {
    msec=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_SEC))
  {
    tm.tm_sec=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_MIN))
  {
    tm.tm_min=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_HOUR))
  {
    tm.tm_hour=GTK_ADJUSTMENT(widget)->value;
  }
  if (msec>=1000) { tm.tm_sec++; msec-=1000; }
  if (msec<0) { tm.tm_sec--; msec+=1000; }
  mktime_ntz(&tm);

  stop=1;
  Set_Adjust(widget,LAB_MSEC,"%d" ,msec);
  Set_Adjust(widget,LAB_SEC,"%d" ,tm.tm_sec);
  Set_Adjust(widget,LAB_MIN,"%d",tm.tm_min);
  Set_Adjust(widget,LAB_HOUR,"%d" ,tm.tm_hour);
  stop=0;
  func(&tm);
}
GtkWidget *Create_TimeArray(char *name,struct tm func(),struct tm tm,int msec)
{
  GtkWidget *w;
  w=Create_ButtonArray(name,Create_TimeArray_func,4,
      SPIN,LAB_HOUR  ,"%d%d%d",tm.tm_hour,-23,23,
      SPIN,LAB_MIN   ,"%d%d%d",tm.tm_min,-1,60,
      SPIN,LAB_SEC   ,"%d%d%d",tm.tm_sec,-1,60,
      SPIN,LAB_MSEC  ,"%d%d%d%d",100,msec,-1000,1000,
      NULL);

  func(&tm);
  gtk_object_set_data(GTK_OBJECT(w),FUNCID,(gpointer)func);
ww=w;
  return w;
}

struct tm abc(struct tm *itm)
{
  static struct tm tm;
  if (itm) tm=*itm;
printf(asctime(&tm));
  return tm;
}
#endif
void Create_preferences_wnd(GtkWidget *widget)
{
  GtkWidget *main_window=gtk_widget_get_toplevel(widget);
  GtkWidget *wnd,*wt[3],*wa[3],*wb,*wc,*wd,*we[5],*wf[4],*wg,*wx,*wy1,*wy2,*wy,*wz;
  char progs[4][100];
  strcpy(progs[0],dbl_prct(db->prog_track));
  strcpy(progs[1],dbl_prct(db->prog_up));
  strcpy(progs[2],dbl_prct(db->prog_down));
  strcpy(progs[3],dbl_prct(db->prog_trackup));

  if (!(wnd=Create_Window(main_window,0,0,"Preferences",NULL))) return;

  if (exist_file(PREF_FILE2))
    wt[1]=Create_Entry(LAB_LOPREF,NULL,PREF_FILE2); 
  else if (exist_file(PREF_FILE))
    wt[1]=Create_Entry(LAB_LOPREF,NULL,PREF_FILE); 
  else
    wt[1]=Create_Entry(LAB_LOPREF,NULL,"Not found."); 

  if (exist_file(db->used_preffile))
    wt[2]=Create_Entry(LAB_GLPREF,NULL,db->used_preffile); 
  else
    wt[2]=Create_Entry(LAB_GLPREF,NULL,"Not found"); 

  wt[0]=Pack("Preference file",'v',wt[1],5,wt[2],5,NULL);

  wa[1]=Create_ButtonArray("Location observer:",callback_pref,2,
    LABEL,LAB_POSLON+1,
    SPIN,LAB_POSLON  ,"%4f%f%f",R2D(db->refpos.lon),-180.,180.,
    LABEL,LAB_POSLAT+1,
    SPIN,LAB_POSLAT  ,"%4f%f%f",R2D(db->refpos.lat),-90.,90.,
    NULL);

  wa[2]=Create_ButtonArray("Visible at:",callback_pref,2,
    LABEL,"Elevation=",
    SPIN,LAB_ELEDET,"%d%d%d",(int)(R2D(db->elev_det)),-5,90,
    NULL);
  wa[1]=Pack(NULL,'v',wa[1],1,wa[2],1,NULL);


  wa[2]=Create_ButtonArray("Time",callback_pref,2,
    LABEL,"Time used for calc. is UTC.",
    LABEL,"",
    LABEL,LAB_OFFSETTM+1,
    SPIN,LAB_OFFSETTM,"%3.2f%3.2f%f%f",0.25,db->time_offset,-23.,23.,
    LABEL,LAB_OFFSETMS+1,
    SPIN,LAB_OFFSETMS,"%3.1f%f%f%f",0.1,db->time_offset_sec,-60.,60.,
    LABEL,LAB_OFFSETUTC+1,
    SPIN,LAB_OFFSETUTC,"%3.2f%f%f%f",0.25,db->utc_offset,-23.,23.,
/*
    TOGGLE,LAB_ISGM,db->is_gmtime,
*/
                         NULL);

//  wa[2]=Create_TimeArray(LAB_OFFSETTM+1,abc,db->tm_offset,0);

  wa[0]=Pack("",'h',wa[1],1,wa[2],1,NULL);

  wb=Create_ButtonArray("Rotorconfig",callback_pref,2,
    LABEL,LAB_RTYPE,
    TOGGLE,LAB_RTYPEB,FALSE,
    LABEL,LAB_XATDISC+1,
    CHECK,LAB_XATDISC,
    NULL);

  wc=Create_ButtonArray("Files",callback_pref,2,
    LABEL,LAB_KEPF+1,
    ENTRY,LAB_KEPF,"%-20s",db->pref_noradfile,
    LABEL,LAB_MAPF+1,
    ENTRY,LAB_MAPF,"%-20s",db->pref_mapfile,
                         0);

  wd=Create_ButtonArray("Programs",callback_pref,2,
    LABEL,LAB_PROGTRACK+1,
    ENTRY,LAB_PROGTRACK  ,"%-30s",progs[0],
    LABEL,LAB_PROGUP+1,
    ENTRY,LAB_PROGUP     ,"%-30s",progs[1],
    LABEL,LAB_PROGDOWN+1,
    ENTRY,LAB_PROGDOWN   ,"%-30s",progs[2],
    LABEL,LAB_PROGTRACKUP+1,
    ENTRY,LAB_PROGTRACKUP,"%-30s",progs[3],
                         0);

  we[1]=Create_Check(LAB_START,callback_pref,FALSE);
  we[2]=Create_Spin(LAB_REPTIM,callback_pref,"%3.1f%3.1f%3.1f",db->reptime,0.1,10000.);
  we[3]=Create_ButtonArray("Antenna direction to output:",callback_pref,1,
                        CHECK,LAB_OUTON,
                        CHECK,LAB_EXTPROG,
                        CHECK,LAB_SERIAL,
                        0);
  we[4]=Create_Toggle(LAB_FREQ,callback_pref,FALSE);
  we[4]=SPack("Receiver freq.:","v",we[4],"f1",NULL);
db->satsel_bottom=FALSE;
#ifdef WERKTNIETGOED
  we2=Create_Optionmenu(LAB_POSSATSEL,callback_pref,(db->satsel_bottom? 1 : 0),
                        LAB_POSSATSEL_R,LAB_POSSATSEL_B,
                        0);

  we2=Pack(NULL,'v',we2,5,NULL);
  we=Pack("",'h',we1,5,we2,5,NULL);
#endif
  we[0]=Pack("",'v',we[1],1,we[2],1,NULL);
  we[3]=Pack("",'v',we[3],1,we[4],1,NULL);
  we[0]=Pack("",'h',we[0],1,we[3],1,NULL);


  wf[1]=Create_ButtonArray("",callback_pref,2,
                        LABEL,LAB_COMPORT+1,
                        SPIN,LAB_COMPORT,"%3d%3d%3d",db->rs232.portnr+PORTMIN,PORTMIN,PORTMAX,
                        LABEL,LAB_COMSPEED+1,
                        SPIN,LAB_COMSPEED,"%3d%3d%3d%3d",9600,db->rs232.speed,9600,19200,
                        LABEL,"parity,stop: fixed",
                        LABEL,"parity=N,stop=1",
                        0);

  {
    char tmp[100],*p;
    *tmp=0;
    for (p=db->rs232.command; *p; p++)
    {
      if (*p=='%') strcat(tmp,"%");
      sprintf(tmp,"%s%c",tmp,*p);
    }
    wf[2]=Create_ButtonArray("",callback_pref,2,
//                        LABEL,"2 numbers:",
//                        LABEL,"azim+elev or x+y",
//                        LABEL,"3 numbers:",
//                        LABEL,"e/w pass + azim+elev",
                        LABEL,LAB_COMMAND+1,
                        ENTRY,LAB_COMMAND,"%-20s",tmp,
/*
                        LABEL,"Examples:",
                        LABEL,"",
                        LABEL,LAB_EXAMPLE1+1,
                        BUTTON,LAB_EXAMPLE1+1,
                        LABEL,LAB_EXAMPLE2+1,
                        BUTTON,LAB_EXAMPLE2,
                        LABEL,LAB_EXAMPLE3+1,
                        BUTTON,LAB_EXAMPLE3,
                        LABEL,LAB_EXAMPLE4+1,
                        BUTTON,LAB_EXAMPLE4,
*/
                        0);
  }
  wf[3]=Create_ButtonArray("Generate RS232 command examples",callback_pref,2,
                        LABEL,"",
                        LABEL,"",
                        LABEL,"2 numbers:",
                        LABEL,"azim+elev or x+y      ",
                        LABEL,"3 numbers:",
                        LABEL,"e/w pass + azim+elev",
                        LABEL,"",
                        LABEL,"",
                        BUTTON,LAB_EXAMPLE1,
                        BUTTON,LAB_EXAMPLE2,
                        BUTTON,LAB_EXAMPLE3,
                        BUTTON,LAB_EXAMPLE4,
                        0);

  wf[2]=Pack("",'v',wf[2],5,wf[3],5,NULL);
  wf[0]=Pack("",'h',wf[1],1,wf[2],1,NULL);

  wg=Create_Spin(LAB_FONTS,callback_pref,"%3d%3d%3d%d",10,db->fontsize,0,200);

/* The notebook */
  wx=Create_Notebook(NULL,GTK_POS_TOP,
                     "Observator"         ,wa[0],
                     "Rotorconfig"        ,wb,
                     "Files"              ,wc,
                     "Programs"           ,wd,
                     "Run options"        ,we[0],
                     "RS232"              ,wf[0],
//                     "Fonts & colors"     ,wg,
                     NULL);

  wy1=Create_Button(LAB_SAVE,callback_pref);
  wy2=Create_Button(LAB_CLOSE,callback_pref);
  wy=SPack("","h",wy1,"fe5",wy2,"fe5",NULL);

  wz=Pack(NULL,'v',wt[0],5,wx,5,wy,5,NULL);

  Set_Button(wz,LAB_RTYPEB,db->use_xy);
  Set_Button(wz,LAB_XATDISC,db->x_at_disc);
  Set_Button(wz,LAB_OUTON,db->out_on);
  Set_Button(wz,LAB_EXTPROG,db->ext_on);
  Set_Button(wz,LAB_START,db->start_now);
  Set_Button(wz,LAB_SERIAL,db->to_serial);
  Set_Adjust(wz,LAB_COMPORT,"%d",db->rs232.portnr+PORTMIN);
  Set_Adjust(wz,LAB_COMSPEED,"%d",db->rs232.speed);
  gtk_container_add(GTK_CONTAINER(wnd),wz);
  gtk_widget_show(wnd);
  
}
