#include <math.h>
#include <string.h>
#include <stdio.h>

#include "sattrack.h"
#include "xtrack_basefunc.h"
//#include "xtrack.h"
//#include "xtrack_func.h"

/*************************************
 * Wild char comparator
 *************************************/
int strcmpwild(char *s,char *v)
{
  while ((*v) && (*s))
  {
    if (*v=='*')
    {
      v++;
      if (!*v) return 0;                   /* "abc"=="abc*" */
      while ((*s) && (strcmpwild(s,v))) s++;
    }
    if ((*v!='?') && (*v!=*s)) return 1;   /* "abc"=="adb" */
    v++; s++;
  }
  if (*s) return 2;                        /* "abc"=="ab" */
  while (*v=='*') v++;
  if (*v) return 3;                        /* "ab"=="abc" */
  return 0;                                /* "abc"=="a*c" enz. */
}    
/* Translate Kepler angles to radians */
void kepler_d2r(KEPLER *kepler)
{
  kepler->inclination=D2R(kepler->inclination);
  kepler->raan=D2R(kepler->raan);
  kepler->perigee=D2R(kepler->perigee);
  kepler->anomaly=D2R(kepler->anomaly);
}

/*************************************
 * Search satellite in NORAD 2 lines format file
 *
 * fp            : file pointer to opened file 
 * satname_tofind: name of satellite to find
 * kepler        : result Kepler data
 *
 * Return: 1 if succeeded
 *         0 if failed (EOF)
 *************************************/
#ifdef XXX
int get_satkeps(FILE *fp,char *satname_tofind,KEPLER *kepler)
{
  char satname_infile[100];

/* Reset Kepler-vars */
  kepler->epoch_year=0;
  kepler->epoch_day=0.;
  kepler->decay_rate=0.;
  kepler->inclination=0.;
  kepler->raan=0.;
  kepler->eccentricity=0.;
  kepler->perigee=0.;
  kepler->anomaly=0.;
  kepler->motion=0.;
  kepler->epoch_rev=0;

  if (!fp) return 0;

/* Search in file requested satellite name. */
  rewind(fp);
  while ((read_norad_next_keps(fp,satname_infile,kepler)))
  {
    if (!strcmp(satname_infile,satname_tofind))
    {
//      kepler_d2r(kepler);
      return 1;
    }
  }
/* Satellite not found! */
  return 0;
}
#endif

/* Wild characters: * and ?, anywhere in string.
    return=0 if match
    return!=0 if no match
*/

SAT *read_msat(FILE *fp,char *satnames_toload,SAT *sat)
{
  KEPLER kepler;
  char satname_infile[100];
  char *satname,satnames[100];
  if (!fp) return NULL;
  rewind(fp);
  while ((read_norad_next_keps(fp,satname_infile,&kepler)))
  {
    strncpy(satnames,satnames_toload,90);
    for (satname=strtok(satnames,";"); satname; satname=strtok(NULL,";"))
    {
      if (!strcmpwild(satname_infile,satname))
      {
        sat=Create_Sat(&sat);
        if (!sat) continue;
        sat->kepler=kepler;
        calc_orbitconst(&sat->kepler,&sat->orbit);
        strcpy(sat->satname,satname_infile);
        if (strstr(sat->satname,"NOAA"))
        {
          sat->orbit.max_sens_angle=D2R(55.4); /* for NOAA */
          sat->orbit.width_original=2048;
          if (sat->lfreq==0.)
          {
            if (strstr(sat->satname,"15")) { sat->lfreq=137.620;  sat->hfreq=1702.5; }
            if (strstr(sat->satname,"16")) { sat->lfreq=0;        sat->hfreq=1698.0; }
            if (strstr(sat->satname,"17")) { sat->lfreq=0;        sat->hfreq=0;      }
            if (strstr(sat->satname,"18")) { sat->lfreq=137.9125; sat->hfreq=1707.0; }
            if (strstr(sat->satname,"19")) { sat->lfreq=137.100;  sat->hfreq=1698.0; }
          }
        }
        else if (strstr(sat->satname,"METEOR"))
        {
          sat->orbit.max_sens_angle=D2R(55.4); /* for Meteor */
          sat->orbit.width_original=1540;
        }
        else if (strstr(sat->satname,"FENG"))
        {
          sat->orbit.max_sens_angle=D2R(55.4); /* for Fengyun */
          sat->orbit.width_original=2048;
        }
        else if (strstr(sat->satname,"SEASTAR??")) /* To find out actual name! */
        {
          sat->orbit.max_sens_angle=D2R(58.3); /* for Seastar */
          sat->orbit.width_original=2048;
        }
        else
          sat->orbit.max_sens_angle=0;         /* No sensor calc */
      }
    }
  }
  return sat;
}

