#ifndef SATTRACK_HDR
#define SATTRACK_HDR

#include <malloc.h>
#include <time.h>
#ifndef USE_GTK
typedef enum {FALSE=0,TRUE} gboolean;
#endif
#ifndef __SCANDIR__
#define  __SCANDIR__
typedef enum {scan_unk=0,N_S,S_N,W_E,E_W} SCANDIR;
#endif

#ifndef PI
#define PI 3.14159265
#endif
#define PIx2 (PI*2.)           /* not present in math.h */
#define D2R(g) ((g)*PI/180.)     /* degree --> radians */
#define R2D(g) ((g)*180./PI)     /* radians --> degree */
#ifndef ABS
#define ABS(a) ((a)<0? (-1*a) : (a))
#endif

typedef enum {satellite=0,sun,moon} SATTYPE;

/*************************************
 * Some constants
 *************************************/
#define G0    9.798                       /* gravity constant */
#define Rearth 6378160.                   /* radius earth (meters) */       
#define Const 9.95                        /* ??? */
#define Earth_offsetangle (98.967440+1)   /* correctie! ??? 9-10-2013: extra cor. +1 */
#define LEN_YEAR 365.2422
#define LEN_LEAPYEAR 366.2422

typedef struct kepler
{
  int    epoch_year;
  float  epoch_day;
  float  decay_rate;
  float  inclination;
  float  raan;
  float  eccentricity;
  float  perigee;
  float  anomaly;
  float  motion;
  long   epoch_rev;
} KEPLER;

typedef struct orbit
{
  gboolean valid;                  /* flag: kepler data valid? */

/* Start time record (used) */
  struct tm start_tm;              /* record start time  */
  int       start_ms;              /* record start time msec */

/* Start time record (satellite) */
  struct tm start_tm_sat;          /* record start time  */
  int       start_ms_sat;          /* record start time msec */

/* Start time record (computer) */
  struct tm start_tm_comp;          /* record start time  */

/* Reference time Kepler data */
  long    ref_time;              /* total reference time  */
  struct tm ref_tm;                /* reference time year ... sec */
  int       ref_ms;                /* reference time msec */

  float height;                    /* sat height [meters] */ 
  int   data_age;
  float loop_time;                 /* time 1 loop [seconds] */
  float k2_n;
  float k2_w;
  float dh_w;

/* Some satellite dependent constants */
  float max_sens_angle;
  SCANDIR scan_hor,scan_ver;       /* Movement of satellite */
  int width_original;

  gboolean use_sattime;
  int offset_gmt;
  int offset_sec;
  float offset_lon;
} ORBIT;

typedef struct epoint
{
  float x,y,z;
  float lon,lat;
} EPOINT;

typedef struct direction
{
  float elev,azim;
  float x,y;
} DIRECTION;

typedef struct sat
{
  struct sat *prev,*next;
  struct dbase *db;
  char satname[40];
  KEPLER kepler;
  ORBIT orbit;
  EPOINT pos;
  DIRECTION dir;
  float max_elev;
  int pass_e1_w0;
  gboolean visible; 
  gboolean selected; 
  float hfreq,lfreq;
  SATTYPE type;
} SAT;

typedef struct track
{
  struct track *prev,*next;
  EPOINT up_pos;
  struct tm up_time;
  struct sat *sat;

  EPOINT down_pos;
  struct tm down_time;

  EPOINT maxelev_pos;
  struct tm maxelev_time;
  float max_elev;
  int pass_e1_w0;
  gboolean going_south;
} TRACK;


#endif
