/**************************************************
 * RCSId: $Id$
 *
 * RGB drawing functions.
 * Project: SGTK
 * Author: R. Alblas
 *
 * History: 
 * $Log$
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include "sgtk.h"

/*************************************
 * Draw a point using RGB (no gtk function for this!??)
 *************************************/
void draw_rgbpoint(RGBI *rgbi,
                   GdkColor *clr,                   /* draw color */
                   int x,int y)
{
  int rgbbux_offset;
  if (!rgbi->rgbbuf) return;
  if ((x<0) || (x>=rgbi->width)) return;
  if ((y<0) || (y>=rgbi->height)) return;
  rgbbux_offset=(y*rgbi->width+x)*3;
  rgbi->rgbbuf[rgbbux_offset+0]=clr->red&0xff;
  rgbi->rgbbuf[rgbbux_offset+1]=clr->green&0xff;
  rgbi->rgbbuf[rgbbux_offset+2]=clr->blue&0xff;
}

/*************************************
 * Get point value from RGB-buffer
 *************************************/
int get_rgbpoint(RGBI *rgbi,GdkColor *clr,int x,int y)
{
  int rgbbux_offset;
  if ((x<0) || (x>=rgbi->width)) return 0;
  if ((y<0) || (y>=rgbi->height)) return 0;

  rgbbux_offset=(y*rgbi->width+x)*3;
  clr->red=rgbi->rgbbuf[rgbbux_offset+0];
  clr->green=rgbi->rgbbuf[rgbbux_offset+1];
  clr->blue=rgbi->rgbbuf[rgbbux_offset+2];
  return 1;
}

/*************************************
 * Draw a line using RGB (no gtk function for this!??)
 *************************************/
void draw_rgbline(RGBI *rgbi,
                  GdkColor *clr,                      /* draw color */
                  int x1,int y1,int x2,int y2)        /* start/end point */

{
  int ix,iy;

  if (ABS(x2-x1) > ABS(y2-y1))
  {
    if (x1>x2)
    {
      ix=x1; x1=x2; x2=ix;
      iy=y1; y1=y2; y2=iy;
    }

    for (ix=x1; ix<=x2; ix++)
    {
      if (x2!=x1)
        iy=(float)(ix-x1)/(float)(x2-x1)*(y2-y1)+y1;
      else
        iy=y2;

/* Test out-of-range */
      if ((ix<0) || (ix>=rgbi->width)) continue;
      if ((iy<0) || (iy>=rgbi->height)) continue;

      draw_rgbpoint(rgbi,clr,ix,iy);
    }
  }
  else
  {
    if (y1>y2)
    {
      ix=x1; x1=x2; x2=ix;
      iy=y1; y1=y2; y2=iy;
    }

    for (iy=y1; iy<=y2; iy++)
    {
      if (y2!=y1)
        ix=(float)(iy-y1)/(float)(y2-y1)*(x2-x1)+x1;
      else
        ix=x2;

/* Test out-of-ramge */
      if ((ix<0) || (ix>=rgbi->width)) continue;
      if ((iy<0) || (iy>=rgbi->height)) continue;

      draw_rgbpoint(rgbi,clr,ix,iy);
    }
  }
}

void draw_rgbrect(RGBI *rgbi,GdkColor *clr,int x,int y,int w,int h)
{
  draw_rgbline(rgbi,clr,x    , y    , x+w-1, y);
  draw_rgbline(rgbi,clr,x+w-1, y    , x+w-1, y+h-1);
  draw_rgbline(rgbi,clr,x+w-1, y+h-1, x    , y+h-1);
  draw_rgbline(rgbi,clr,x    , y+h-1, x    , y);
}

/*************************************
 * ASCII bitmap for draw_rgbstring
 *************************************/
static char ta[][8]={ {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
                      {0x30,0x78,0x78,0x78,0x30,0x00,0x30,0x00},
                      {0x6c,0x6c,0x6c,0x00,0x00,0x00,0x00,0x00},
                      {0x6c,0x6c,0xfe,0x6c,0xfe,0x6c,0x6c,0x00},
                      {0x30,0x7c,0xc0,0x78,0x0c,0xf8,0x30,0x00},
                      {0x00,0xc6,0xcc,0x18,0x30,0x66,0xc6,0x00},
                      {0x38,0x6c,0x38,0x76,0xdc,0xcc,0x76,0x00},
                      {0x60,0x60,0xc0,0x00,0x00,0x00,0x00,0x00},
                      {0x18,0x30,0x60,0x60,0x60,0x30,0x18,0x00},
                      {0x60,0x30,0x18,0x18,0x18,0x30,0x60,0x00},
                      {0x00,0x66,0x3c,0xfe,0x3c,0x66,0x00,0x00},
                      {0x00,0x30,0x30,0xfc,0x30,0x30,0x00,0x00},
                      {0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x60},
                      {0x00,0x00,0x00,0xfc,0x00,0x00,0x00,0x00},
                      {0x00,0x00,0x00,0x00,0x00,0x30,0x30,0x00},
                      {0x06,0x0c,0x18,0x30,0x60,0xc0,0x80,0x00},
                      {0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00},
                      {0x30,0x70,0x30,0x30,0x30,0x30,0xfc,0x00},
                      {0x78,0xcc,0x0c,0x38,0x60,0xcc,0xfc,0x00},
                      {0x78,0xcc,0x0c,0x38,0x0c,0xcc,0x78,0x00}, 
                      {0x1c,0x3c,0x6c,0xcc,0xfe,0x0c,0x1e,0x00},
                      {0xfc,0xc0,0xf8,0x0c,0x0c,0xcc,0x78,0x00},
                      {0x38,0x60,0xc0,0xf8,0xcc,0xcc,0x78,0x00},
                      {0xfc,0xcc,0x0c,0x18,0x30,0x30,0x30,0x00},
                      {0x78,0xcc,0xcc,0x78,0xcc,0xcc,0x78,0x00},
                      {0x78,0xcc,0xcc,0x7c,0x0c,0x18,0x70,0x00},
                      {0x00,0x30,0x30,0x00,0x00,0x30,0x30,0x00},
                      {0x00,0x30,0x30,0x00,0x00,0x30,0x30,0x60},
                      {0x18,0x30,0x60,0xc0,0x60,0x30,0x18,0x00},
                      {0x00,0x00,0xfc,0x00,0x00,0xfc,0x00,0x00},
                      {0x60,0x30,0x18,0x0c,0x18,0x30,0x60,0x00},
                      {0x78,0xcc,0x0c,0x18,0x30,0x00,0x30,0x00},
                      {0x7c,0xc6,0xde,0xde,0xde,0xc0,0x78,0x00},
                      {0x38,0x6c,0xc6,0xc6,0xfe,0xc6,0xc6,0x00},
                      {0xfc,0xc6,0xc6,0xfc,0xc6,0xc6,0xfc,0x00},
                      {0x7c,0xc6,0xc6,0xc0,0xc0,0xc6,0x7c,0x00}, 
                      {0xf8,0xcc,0xc6,0xc6,0xc6,0xcc,0xf8,0x00},
                      {0xfe,0xc0,0xc0,0xfc,0xc0,0xc0,0xfe,0x00},
                      {0xfe,0xc0,0xc0,0xfc,0xc0,0xc0,0xc0,0x00},
                      {0x7c,0xc6,0xc0,0xce,0xc6,0xc6,0x7e,0x00},
                      {0xc6,0xc6,0xc6,0xfe,0xc6,0xc6,0xc6,0x00},
                      {0x78,0x30,0x30,0x30,0x30,0x30,0x78,0x00},
                      {0x1e,0x06,0x06,0x06,0xc6,0xc6,0x7c,0x00},
                      {0xc6,0xcc,0xd8,0xf0,0xd8,0xcc,0xc6,0x00},
                      {0xc0,0xc0,0xc0,0xc0,0xc0,0xc0,0xfe,0x00},
                      {0xc6,0xee,0xfe,0xd6,0xc6,0xc6,0xc6,0x00},
                      {0xc6,0xe6,0xf6,0xde,0xce,0xc6,0xc6,0x00},
                      {0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00},
                      {0xfc,0xc6,0xc6,0xfc,0xc0,0xc0,0xc0,0x00},
                      {0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x06},
                      {0xfc,0xc6,0xc6,0xfc,0xc6,0xc6,0xc6,0x00},
                      {0x78,0xcc,0x60,0x30,0x18,0xcc,0x78,0x00},
                      {0xfc,0x30,0x30,0x30,0x30,0x30,0x30,0x00},
                      {0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,0x7c,0x00},
                      {0xc6,0xc6,0xc6,0xc6,0xc6,0x6c,0x38,0x00},
                      {0xc6,0xc6,0xc6,0xd6,0xfe,0xee,0xc6,0x00}, 
                      {0xc6,0xc6,0x6c,0x38,0x6c,0xc6,0xc6,0x00},
                      {0xc2,0xc2,0x66,0x3c,0x18,0x18,0x18,0x00},
                      {0xfe,0x0c,0x18,0x30,0x60,0xc0,0xfe,0x00},
                      {0x3c,0x30,0x30,0x30,0x30,0x30,0x3c,0x00},
                      {0xc0,0x60,0x30,0x18,0x0c,0x06,0x02,0x00},
                      {0x3c,0x0c,0x0c,0x0c,0x0c,0x0c,0x3c,0x00},
                      {0x00,0x38,0x6c,0xc6,0x00,0x00,0x00,0x00},
                      {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfe},
                      {0x30,0x30,0x18,0x00,0x00,0x00,0x00,0x00},
                      {0x00,0x00,0x7c,0x06,0x7e,0xc6,0x7e,0x00},
                      {0xc0,0xc0,0xfc,0xc6,0xc6,0xe6,0xdc,0x00},
                      {0x00,0x00,0x7c,0xc6,0xc0,0xc0,0x7e,0x00},
                      {0x06,0x06,0x7e,0xc6,0xc6,0xce,0x76,0x00},
                      {0x00,0x00,0x7c,0xc6,0xfe,0xc0,0x7e,0x00},
                      {0x1e,0x30,0x7c,0x30,0x30,0x30,0x30,0x00},
                      {0x00,0x00,0x7e,0xc6,0xce,0x76,0x06,0x7c},
                      {0xc0,0xc0,0xfc,0xc6,0xc6,0xc6,0xc6,0x00},
                      {0x18,0x00,0x38,0x18,0x18,0x18,0x3c,0x00},
                      {0x18,0x00,0x38,0x18,0x18,0x18,0x18,0xf0},
                      {0xc0,0xc0,0xcc,0xd8,0xf0,0xd8,0xcc,0x00},
                      {0x38,0x18,0x18,0x18,0x18,0x18,0x3c,0x00},
                      {0x00,0x00,0xcc,0xfe,0xd6,0xc6,0xc6,0x00},
                      {0x00,0x00,0xfc,0xc6,0xc6,0xc6,0xc6,0x00},
                      {0x00,0x00,0x7c,0xc6,0xc6,0xc6,0x7c,0x00},
                      {0x00,0x00,0xfc,0xc6,0xc6,0xe6,0xdc,0xc0},
                      {0x00,0x00,0x7e,0xc6,0xc6,0xce,0x76,0x06},
                      {0x00,0x00,0x6e,0x70,0x60,0x60,0x60,0x00},
                      {0x00,0x00,0x7c,0xc0,0x7c,0x06,0xfc,0x00},
                      {0x30,0x30,0x7c,0x30,0x30,0x30,0x1c,0x00},
                      {0x00,0x00,0xc6,0xc6,0xc6,0xc6,0x7e,0x00},
                      {0x00,0x00,0xc6,0xc6,0xc6,0x6c,0x38,0x00},
                      {0x00,0x00,0xc6,0xc6,0xd6,0xfe,0x6c,0x00},
                      {0x00,0x00,0xc6,0x6c,0x38,0x6c,0xc6,0x00},
                      {0x00,0x00,0xc6,0xc6,0xce,0x76,0x06,0x7c},
                      {0x00,0x00,0xfc,0x18,0x30,0x60,0xfc,0x00},
                      {0x1c,0x30,0x30,0xe0,0x30,0x30,0x1c,0x00},
                      {0x18,0x18,0x18,0x00,0x18,0x18,0x18,0x00},
                      {0xe0,0x30,0x30,0x1c,0x30,0x30,0xe0,0x00},
                      {0x76,0xdc,0x00,0x00,0x00,0x00,0x00,0x00}
                    };

/*************************************
 * Draw a string using RGB (no gtk function for this!??)
 *************************************/
void draw_rgbstring(RGBI *rgbi,
                    GdkColor *clr,                   /* draw color */
                    int x,int y,
                    char *s)
{
  int x1,y1,m=0,n;
  int h,v;

  while (*s)
  {
    n=(*s)-' '; if (n>95) return;
    for (y1=0; y1<8; y1++) 
    {
      v=y+y1;
      if (v>=rgbi->height) continue;
      for (x1=0; x1<7; x1++)
      {
        h=x+x1+m*8;
        if (h>=rgbi->width) continue;
        if (ta[n][y1]&(0x80>>x1))
          draw_rgbpoint(rgbi,clr,x+x1+m*8,y+y1);
      }
    }
    m++; s++;
  }
}

void get_strline(char *s,int y,char *l,int slen)
{
  int m,x,n;
  *l=0;
  for (m=0; *s; s++,m++)
  {
    n=(*s)-' '; if (n>95) return;
    if (m*8+8>slen) return;
    for (x=0; x<8; x++)
    {
      if (ta[n][y]&(0x80>>x)) l[x+m*8]=0xff; else l[x+m*8]=0;
    }
  }
}
