#define USE_GTK

#define DEF_NORADFILE "norad.txt"
#define DEF_SAT "NOAA-16"
#define DEF_MAPFILE "earthmap.gif"
#define MAX_FILENAME_LENGTH 512

#ifdef WINDHOOS
#define TYPE_COMMAND "type"
#else
#define TYPE_COMMAND "cat"
#endif

#include <float.h>
#ifndef FLT_MAX
#include <float.h>
#endif

#include <time.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>

//typedef enum {mode_unk=0,search_up,search_down,search_maxelev} SEARCH_MODE;

#ifndef MAX
#define MAX(a,b) ((a)>(b)? (a) : (b))
#define MIN(a,b) ((a)<(b)? (a) : (b))
#endif


#ifdef USE_GTK
#include "gtk/gtk.h"
#include "gdk/gdkkeysyms.h"
#include "sgtk.h"
#include "sattrack.h"

#else
#ifndef gint32
typedef long gint32;
#endif

#ifndef gboolean
typedef enum {FALSE=0,TRUE} gboolean;
#endif
#endif

#include "sattrack.h"

#define LENSATNAME 40
#define FACT 2.0

#define SEL_LAB "Sel"
#define VIS_LAB "Vis"

typedef enum { unknown_filetype,SUN,TIFF,GIF,JPEG } FILE_TYPE;

#define MAX_PCICLR 4096
typedef struct clrmap
{
  int size;
  int red[MAX_PCICLR],green[MAX_PCICLR],blue[MAX_PCICLR];
} CLRMAP;

typedef struct rs232_items
{
  int portnr;
  int speed;
  char command[100];
} RS232_ITEMS;

typedef struct dbase
{
  EPOINT refpos;
  EPOINT sun;
  SAT *sat;
  SAT *sat_sel;
  SAT *next_sat;

  struct tm glob_tm;
  int glob_ds;        // deciseconds
  struct tm genstart_tm;
  struct tm genrange_tm;

  char satname[40];
  char norad_file[MAX_FILENAME_LENGTH];
  char pref_noradfile[MAX_FILENAME_LENGTH];
  char map_file[MAX_FILENAME_LENGTH];
  char pref_mapfile[MAX_FILENAME_LENGTH];
  gboolean x_at_disc;
  FILE *fp_norad;
/*
  int hour_offset;
*/
  struct tm tm_offset;
  float time_offset;
  float time_offset_sec;
  float utc_offset;
  char ofile[50];
  float ppl;             /* zoom */
  float fact;             /* zoom */
  float zx,zy;
  int ox,oy;
  gboolean redraw;
  gboolean redrawing;
  gboolean setscrolflag;
  gboolean use_xy;
  float elev_hor;
  float elev_det;
  struct track *track;
  gboolean hres;
  gboolean start_now;
  gboolean out_on;
  gboolean ext_on;
  gboolean to_serial;
  gboolean satsel_bottom;
  gboolean auto_select;
  gboolean is_gmtime;

  char prog_up[100];
  char prog_down[100];
  char prog_track[100];
  char prog_trackup[100];

  int ps_x_offset,ps_y_offset;
  int ps_max_h;
  int ps_max_t;
  float reptime;
  struct rs232_items rs232;
  char used_preffile[MAX_FILENAME_LENGTH];  /* preffile used to read from */

  char *cur_dir;
  char *home_dir;
  char *prog_dir;

  int fontsize;
} DBASE;

#ifdef XXXX
typedef struct track
{
  struct track *prev,*next;
  EPOINT up_pos;
  struct tm up_time;

  EPOINT down_pos;
  struct tm down_time;

  EPOINT maxelev_pos;
  struct tm maxelev_time;
  float max_elev;

  gboolean going_south;
} TRACK;
#endif



typedef struct picinfo
{
  char file_name[MAX_FILENAME_LENGTH]; /* name of current file */
  FILE *fp;                        /* filepointer; NULL if no file opened */
  char sat_name[LENSATNAME];       /* name satellite */

  int width,height,depth;          /* Size picture */
  float alpha_max;                 /* satellite sensor angle */
  long pic_strt;                   /* pic starts at this byte number */

  int width_original;              /* Size original pic */
  int ox,oy;                       /* Offset to original pic */
  int lines_per_sec;               /* lines per second */

  SCANDIR scan_hor,scan_ver;       /* Movement of satellite */
  struct tm picture_time;          /* Time of pic (not always time of record) */

  KEPLER kepler;                   /* Keplerian elements of sat orbit */
  ORBIT orbit;                     /* Orbit vars */
} PICINFO;

#define Rewind(str) while ((str) && (str->prev)) str=str->prev

void yday2mday_mon(struct tm *tmref);
gint32 mktime_ntz(struct tm *tm);
float alpha_2_beta(float ,SCANDIR,int ,int ,float );
void calceleazim(EPOINT *,ORBIT *,EPOINT *,DIRECTION *,gboolean);
void Read_Prefs(char *,DBASE *);
void Save_Prefs(DBASE *,gboolean);
void Create_preferences_wnd(GtkWidget *);


